/* ***********************************************************
 * This file was automatically generated on 2012-05-24.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling up to seven servos
	/// </summary>
	public class BrickServo : Device 
	{
		private static byte FUNCTION_ENABLE = 1;
		private static byte FUNCTION_DISABLE = 2;
		private static byte FUNCTION_IS_ENABLED = 3;
		private static byte FUNCTION_SET_POSITION = 4;
		private static byte FUNCTION_GET_POSITION = 5;
		private static byte FUNCTION_GET_CURRENT_POSITION = 6;
		private static byte FUNCTION_SET_VELOCITY = 7;
		private static byte FUNCTION_GET_VELOCITY = 8;
		private static byte FUNCTION_GET_CURRENT_VELOCITY = 9;
		private static byte FUNCTION_SET_ACCELERATION = 10;
		private static byte FUNCTION_GET_ACCELERATION = 11;
		private static byte FUNCTION_SET_OUTPUT_VOLTAGE = 12;
		private static byte FUNCTION_GET_OUTPUT_VOLTAGE = 13;
		private static byte FUNCTION_SET_PULSE_WIDTH = 14;
		private static byte FUNCTION_GET_PULSE_WIDTH = 15;
		private static byte FUNCTION_SET_DEGREE = 16;
		private static byte FUNCTION_GET_DEGREE = 17;
		private static byte FUNCTION_SET_PERIOD = 18;
		private static byte FUNCTION_GET_PERIOD = 19;
		private static byte FUNCTION_GET_SERVO_CURRENT = 20;
		private static byte FUNCTION_GET_OVERALL_CURRENT = 21;
		private static byte FUNCTION_GET_STACK_INPUT_VOLTAGE = 22;
		private static byte FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE = 23;
		private static byte FUNCTION_SET_MINIMUM_VOLTAGE = 24;
		private static byte FUNCTION_GET_MINIMUM_VOLTAGE = 25;
		private static byte CALLBACK_UNDER_VOLTAGE = 26;
		private static byte CALLBACK_POSITION_REACHED = 27;
		private static byte CALLBACK_VELOCITY_REACHED = 28;


		/// <summary>
		///  This callback is triggered when the input voltage drops below the value set by
		///  <see cref="Tinkerforge.BrickServo.SetMinimumVoltage"/>. The parameter is the current voltage given
		///  in mV.
		/// </summary>
		public delegate void UnderVoltage(ushort voltage);

		/// <summary>
		///  This callback is triggered when a position set by <see cref="Tinkerforge.BrickServo.SetPosition"/>
		///  is reached. The parameters are the servo and the position that is reached.
		///  
		///  <note>
		///   Since we can't get any feedback from the servo, this only works if the
		///   velocity (see <see cref="Tinkerforge.BrickServo.SetVelocity"/>) is set smaller or equal to the
		///   maximum velocity of the servo. Otherwise the servo will lag behind the
		///   control value and the callback will be triggered too early.
		///  </note>
		/// </summary>
		public delegate void PositionReached(byte servoNum, short position);

		/// <summary>
		///  This callback is triggered when a velocity set by <see cref="Tinkerforge.BrickServo.SetVelocity"/>
		///  is reached. The parameters are the servo and the velocity that is reached.
		///  
		///  <note>
		///   Since we can't get any feedback from the servo, this only works if the
		///   acceleration (see <see cref="Tinkerforge.BrickServo.SetAcceleration"/>) is set smaller or equal to the
		///   maximum acceleration of the servo. Otherwise the servo will lag behind the
		///   control value and the callback will be triggered too early.
		///  </note>
		/// </summary>
		public delegate void VelocityReached(byte servoNum, short velocity);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickServo(string uid) : base(uid) 
		{
			this.expectedName = "Servo Brick";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_UNDER_VOLTAGE] = new MessageCallback(CallbackUnderVoltage);
			messageCallbacks[CALLBACK_POSITION_REACHED] = new MessageCallback(CallbackPositionReached);
			messageCallbacks[CALLBACK_VELOCITY_REACHED] = new MessageCallback(CallbackVelocityReached);
		}

		/// <summary>
		///  Enables a servo (0 to 6). If a servo is enabled, the configured position,
		///  velocity, acceleration, etc. are applied immediately.
		/// </summary>
		public void Enable(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_ENABLE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Disables a servo (0 to 6). Disabled servos are not driven at all, i.e. a
		///  disabled servo will not hold its position if a load is applied.
		/// </summary>
		public void Disable(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_DISABLE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns true if the specified servo is enabled, false otherwise.
		/// </summary>
		public bool IsEnabled(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_IS_ENABLED, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_IS_ENABLED, out answer);

			return LEConverter.BoolFrom(4, answer);
		}

		/// <summary>
		///  Sets the position in °/100 for the specified servo. 
		///  
		///  The default range of the position is -9000 to 9000, but it can be specified
		///  according to your servo with <see cref="Tinkerforge.BrickServo.SetDegree"/>.
		///  
		///  If you want to control a linear servo or RC brushless motor controller or
		///  similar with the Servo Brick, you can also define lengths or speeds with
		///  <see cref="Tinkerforge.BrickServo.SetDegree"/>.
		/// </summary>
		public void SetPosition(byte servoNum, short position)
		{
			byte[] data_ = new byte[7];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_POSITION, 1, data_);
			LEConverter.To((ushort)7, 2, data_);
			LEConverter.To(servoNum, 4, data_);
			LEConverter.To(position, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the position of the specified servo as set by <see cref="Tinkerforge.BrickServo.SetPosition"/>.
		/// </summary>
		public short GetPosition(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_POSITION, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_POSITION, out answer);

			return LEConverter.ShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the *current* position of the specified servo. This may not be the
		///  value of <see cref="Tinkerforge.BrickServo.SetPosition"/> if the servo is currently approaching a
		///  position goal.
		/// </summary>
		public short GetCurrentPosition(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_POSITION, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CURRENT_POSITION, out answer);

			return LEConverter.ShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the maximum velocity of the specified servo in °/100s. The velocity
		///  is accelerated according to the value set by <see cref="Tinkerforge.BrickServo.SetAcceleration"/>.
		///  
		///  The minimum velocity is 0 (no movement) and the maximum velocity is 65535.
		///  With a value of 65535 the position will be set immediately (no velocity).
		///  
		///  The default value is 65535.
		/// </summary>
		public void SetVelocity(byte servoNum, ushort velocity)
		{
			byte[] data_ = new byte[7];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_VELOCITY, 1, data_);
			LEConverter.To((ushort)7, 2, data_);
			LEConverter.To(servoNum, 4, data_);
			LEConverter.To(velocity, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the velocity of the specified servo as set by <see cref="Tinkerforge.BrickServo.SetVelocity"/>.
		/// </summary>
		public ushort GetVelocity(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_VELOCITY, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_VELOCITY, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the *current* velocity of the specified servo. This may not be the
		///  value of <see cref="Tinkerforge.BrickServo.SetVelocity"/> if the servo is currently approaching a
		///  velocity goal.
		/// </summary>
		public ushort GetCurrentVelocity(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_CURRENT_VELOCITY, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_CURRENT_VELOCITY, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the acceleration of the specified servo in °/100s².
		///  
		///  The minimum acceleration is 1 and the maximum acceleration is 65535.
		///  With a value of 65535 the velocity will be set immediately (no acceleration).
		///  
		///  The default value is 65535.
		/// </summary>
		public void SetAcceleration(byte servoNum, ushort acceleration)
		{
			byte[] data_ = new byte[7];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_ACCELERATION, 1, data_);
			LEConverter.To((ushort)7, 2, data_);
			LEConverter.To(servoNum, 4, data_);
			LEConverter.To(acceleration, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the acceleration for the specified servo as set by 
		///  <see cref="Tinkerforge.BrickServo.SetAcceleration"/>.
		/// </summary>
		public ushort GetAcceleration(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_ACCELERATION, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_ACCELERATION, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the output voltages with which the servos are driven in mV.
		///  The minimum output voltage is 5000mV and the maximum output voltage is 
		///  9000mV.
		///  
		///  <note>
		///    We recommend that you set this value to the maximum voltage that is
		///    specified for your servo, most servos achieve their maximum force only
		///    with high voltages.
		///  </note>
		///  
		///  The default value is 5000.
		/// </summary>
		public void SetOutputVoltage(ushort voltage)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_OUTPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(voltage, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the output voltage as specified by <see cref="Tinkerforge.BrickServo.SetOutputVoltage"/>.
		/// </summary>
		public ushort GetOutputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_OUTPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_OUTPUT_VOLTAGE, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the minimum and maximum pulse width of the specified servo in µs.
		///  
		///  Usually, servos are controlled with a 
		///  `PWM &lt;http://en.wikipedia.org/wiki/Pulse-width_modulation&gt;`_, whereby the 
		///  length of the pulse controls the position of the servo. Every servo has
		///  different minimum and maximum pulse widths, these can be specified with
		///  this function.
		///  
		///  If you have a datasheet for your servo that specifies the minimum and
		///  maximum pulse width, you should set the values accordingly. If your servo
		///  comes without any datasheet you have to find the values via trial and error.
		///  
		///  Both values have a range from 1 to 65535 (unsigned 16 bit integer). The
		///  minimum must be smaller than the maximum.
		///  
		///  The default values are 1000µs (1ms) and 2000µs (2ms) for minimum and 
		///  maximum pulse width.
		/// </summary>
		public void SetPulseWidth(byte servoNum, ushort min, ushort max)
		{
			byte[] data_ = new byte[9];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_PULSE_WIDTH, 1, data_);
			LEConverter.To((ushort)9, 2, data_);
			LEConverter.To(servoNum, 4, data_);
			LEConverter.To(min, 5, data_);
			LEConverter.To(max, 7, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the minimum and maximum pulse width for the specified servo as set by
		///  <see cref="Tinkerforge.BrickServo.SetPulseWidth"/>.
		/// </summary>
		public void GetPulseWidth(byte servoNum, out ushort min, out ushort max)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_PULSE_WIDTH, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_PULSE_WIDTH, out answer);

			min = LEConverter.UShortFrom(4, answer);
			max = LEConverter.UShortFrom(6, answer);
		}

		/// <summary>
		///  Sets the minimum and maximum degree for the specified servo (by default
		///  given as °/100).
		///  
		///  This only specifies the abstract values between which the minimum and maximum
		///  pulse width is scaled. For example: If you specify a pulse width of 1000µs
		///  to 2000µs and a degree range of -90° to 90°, a call of <see cref="Tinkerforge.BrickServo.SetPosition"/>
		///  with 0 will result in a pulse width of 1500µs 
		///  (-90° = 1000µs, 90° = 2000µs, etc.).
		///  
		///  Possible usage:
		///  
		///   * The datasheet of your servo specifies a range of 200° with the middle position at 110°. In this case you can set the minimum to -9000 and the maximum to 11000.
		///   * You measure a range of 220° on your servo and you don't have or need a middle position. In this case you can set the minimum to 0 and the maximum to 22000.
		///   * You have a linear servo with a drive length of 20cm, In this case you could set the minimum to 0 and the maximum to 20000. Now you can set the Position with <see cref="Tinkerforge.BrickServo.SetPosition"/> with a resolution of cm/100. Also the velocity will have a resolution of cm/100s and the acceleration will have a resolution of cm/100s².
		///   * You don't care about units and just want the highest possible resolution. In this case you should set the minimum to -32767 and the maximum to 32767.
		///   * You have a brushless motor with a maximum speed of 10000 rpm and want to control it with a RC brushless motor controller. In this case you can set the minimum to 0 and the maximum to 10000. <see cref="Tinkerforge.BrickServo.SetPosition"/> now controls the rpm.
		///  
		///  Both values have a possible range from -32767 to 32767 
		///  (signed 16 bit integer). The minimum must be smaller than the maximum.
		///  
		///  The default values are -9000 and 9000 for the minimum and maximum degree.
		/// </summary>
		public void SetDegree(byte servoNum, short min, short max)
		{
			byte[] data_ = new byte[9];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_DEGREE, 1, data_);
			LEConverter.To((ushort)9, 2, data_);
			LEConverter.To(servoNum, 4, data_);
			LEConverter.To(min, 5, data_);
			LEConverter.To(max, 7, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the minimum and maximum degree for the specified servo as set by
		///  <see cref="Tinkerforge.BrickServo.SetDegree"/>.
		/// </summary>
		public void GetDegree(byte servoNum, out short min, out short max)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_DEGREE, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_DEGREE, out answer);

			min = LEConverter.ShortFrom(4, answer);
			max = LEConverter.ShortFrom(6, answer);
		}

		/// <summary>
		///  Sets the period of the specified servo in µs.
		///  
		///  Usually, servos are controlled with a 
		///  `PWM &lt;http://en.wikipedia.org/wiki/Pulse-width_modulation&gt;`_. Different
		///  servos expect PWMs with different periods. Most servos run well with a 
		///  period of about 20ms.
		///  
		///  If your servo comes with a datasheet that specifies a period, you should
		///  set it accordingly. If you don't have a datasheet and you have no idea
		///  what the correct period is, the default value (19.5ms) will most likely
		///  work fine. 
		///  
		///  The minimum possible period is 2000µs and the maximum is 65535µs.
		///  
		///  The default value is 19.5ms (19500µs).
		/// </summary>
		public void SetPeriod(byte servoNum, ushort period)
		{
			byte[] data_ = new byte[7];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_PERIOD, 1, data_);
			LEConverter.To((ushort)7, 2, data_);
			LEConverter.To(servoNum, 4, data_);
			LEConverter.To(period, 5, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the period for the specified servo as set by <see cref="Tinkerforge.BrickServo.SetPeriod"/>.
		/// </summary>
		public ushort GetPeriod(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_PERIOD, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_PERIOD, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the current consumption of the specified servo in mA.
		/// </summary>
		public ushort GetServoCurrent(byte servoNum)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_SERVO_CURRENT, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(servoNum, 4, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_SERVO_CURRENT, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the current consumption of all servos together in mA.
		/// </summary>
		public ushort GetOverallCurrent()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_OVERALL_CURRENT, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_OVERALL_CURRENT, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the stack input voltage in mV. The stack input voltage is the
		///  voltage that is supplied via the stack, i.e. it is given by a 
		///  Step-Down or Step-Up Power Supply.
		/// </summary>
		public ushort GetStackInputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STACK_INPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_STACK_INPUT_VOLTAGE, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Returns the external input voltage in mV. The external input voltage is
		///  given via the black power input connector on the Servo Brick. 
		///   
		///  If there is an external input voltage and a stack input voltage, the motor
		///  will be driven by the external input voltage. If there is only a stack 
		///  voltage present, the motor will be driven by this voltage.
		///  
		///  <note type="caution">
		///    This means, if you have a high stack voltage and a low external voltage,
		///    the motor will be driven with the low external voltage. If you then remove
		///    the external connection, it will immediately be driven by the high
		///    stack voltage
		///  </note>
		/// </summary>
		public ushort GetExternalInputVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_EXTERNAL_INPUT_VOLTAGE, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Sets the minimum voltage in mV, below which the <see cref="Tinkerforge.BrickServo.UnderVoltage"/> callback
		///  is triggered. The minimum possible value that works with the Servo Brick is 5V.
		///  You can use this function to detect the discharge of a battery that is used
		///  to drive the stepper motor. If you have a fixed power supply, you likely do 
		///  not need this functionality.
		///  
		///  The default value is 5V (5000mV).
		/// </summary>
		public void SetMinimumVoltage(ushort voltage)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MINIMUM_VOLTAGE, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(voltage, 4, data_);

			sendOneWayMessage(data_);
		}

		/// <summary>
		///  Returns the minimum voltage as set by <see cref="Tinkerforge.BrickServo.SetMinimumVoltage"/>
		/// </summary>
		public ushort GetMinimumVoltage()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MINIMUM_VOLTAGE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] answer;
			sendReturningMessage(data_, FUNCTION_GET_MINIMUM_VOLTAGE, out answer);

			return LEConverter.UShortFrom(4, answer);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void IsEnabled(byte servoNum, out bool enabled)
		{
			enabled = IsEnabled(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetPosition(byte servoNum, out short position)
		{
			position = GetPosition(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentPosition(byte servoNum, out short position)
		{
			position = GetCurrentPosition(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetVelocity(byte servoNum, out ushort velocity)
		{
			velocity = GetVelocity(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetCurrentVelocity(byte servoNum, out ushort velocity)
		{
			velocity = GetCurrentVelocity(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAcceleration(byte servoNum, out ushort acceleration)
		{
			acceleration = GetAcceleration(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetOutputVoltage(out ushort voltage)
		{
			voltage = GetOutputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetPeriod(byte servoNum, out ushort period)
		{
			period = GetPeriod(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetServoCurrent(byte servoNum, out ushort current)
		{
			current = GetServoCurrent(servoNum);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetOverallCurrent(out ushort current)
		{
			current = GetOverallCurrent();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetStackInputVoltage(out ushort voltage)
		{
			voltage = GetStackInputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetExternalInputVoltage(out ushort voltage)
		{
			voltage = GetExternalInputVoltage();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetMinimumVoltage(out ushort voltage)
		{
			voltage = GetMinimumVoltage();
		}

		internal int CallbackUnderVoltage(byte[] data_)
		{
			ushort voltage = LEConverter.UShortFrom(4, data_);

			((UnderVoltage)callbacks[CALLBACK_UNDER_VOLTAGE])(voltage);
			return 6;
		}

		internal int CallbackPositionReached(byte[] data_)
		{
			byte servoNum = LEConverter.ByteFrom(4, data_);
			short position = LEConverter.ShortFrom(5, data_);

			((PositionReached)callbacks[CALLBACK_POSITION_REACHED])(servoNum, position);
			return 7;
		}

		internal int CallbackVelocityReached(byte[] data_)
		{
			byte servoNum = LEConverter.ByteFrom(4, data_);
			short velocity = LEConverter.ShortFrom(5, data_);

			((VelocityReached)callbacks[CALLBACK_VELOCITY_REACHED])(servoNum, velocity);
			return 7;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(UnderVoltage))
			{
				callbacks[CALLBACK_UNDER_VOLTAGE] = d;
			}
			else if(d.GetType() == typeof(PositionReached))
			{
				callbacks[CALLBACK_POSITION_REACHED] = d;
			}
			else if(d.GetType() == typeof(VelocityReached))
			{
				callbacks[CALLBACK_VELOCITY_REACHED] = d;
			}
		}
	}
}
